@@ -25,15 +25,17 @@ CREATE INDEX files_valid ON files (valid);
 
 CREATE TABLE data
 (
-    identity        BLOB PRIMARY KEY NOT NULL,
-    entity          BLOB             NOT NULL,
-    attribute       VARCHAR          NOT NULL,
-    value_str       VARCHAR,
-    value_num       NUMERIC,
-    immutable       BOOLEAN          NOT NULL
+    identity          BLOB PRIMARY KEY NOT NULL,
+    entity            BLOB             NOT NULL,
+    entity_searchable VARCHAR,
+    attribute         VARCHAR          NOT NULL,
+    value_str         VARCHAR,
+    value_num         NUMERIC,
+    immutable         BOOLEAN          NOT NULL
 );
 
 CREATE INDEX data_entity ON data (entity);
+CREATE INDEX data_entity_searchable ON data (entity_searchable);
 CREATE INDEX data_attribute ON data (attribute);
 CREATE INDEX data_value_str ON data (value_str);
 CREATE INDEX data_value_num ON data (value_num);
