@@ -1,5 +1,5 @@
 use crate::addressing::Address;
-use crate::database::entry::{EntryValue, InvariantEntry};
+use crate::database::entry::{InvariantEntry};
 
 pub const TYPE_TYPE_VAL: &str = "TYPE";
 pub const TYPE_BASE_ATTR: &str = "TYPE";
@@ -12,16 +12,17 @@ pub const HIER_TYPE_VAL: &str = "HIER";
 pub const HIER_HAS_ATTR: &str = "HAS";
 
 pub const LABEL_ATTR: &str = "LBL";
+pub const ADDED_ATTR: &str = "ADDED";
 
 lazy_static! {
     pub static ref TYPE_INVARIANT: InvariantEntry = InvariantEntry {
         attribute: String::from(TYPE_BASE_ATTR),
-        value: EntryValue::String(String::from(TYPE_TYPE_VAL)),
+        value: TYPE_TYPE_VAL.into(),
     };
     pub static ref TYPE_ADDR: Address = TYPE_INVARIANT.entity().unwrap();
     pub static ref HIER_INVARIANT: InvariantEntry = InvariantEntry {
         attribute: String::from(TYPE_BASE_ATTR),
-        value: EntryValue::String(String::from(HIER_TYPE_VAL)),
+        value: HIER_TYPE_VAL.into(),
     };
     pub static ref HIER_ADDR: Address = HIER_INVARIANT.entity().unwrap();
 }
