@@ -98,7 +98,7 @@ pub fn list_roots(connection: &UpEndConnection) -> Result<Vec<Address>> {
         connection.query(Query::SingleQuery(QueryPart::Matches(EntryQuery {
             entity: QueryComponent::Any,
             attribute: QueryComponent::Exact(IS_OF_TYPE_ATTR.to_string()),
-            value: QueryComponent::Exact(EntryValue::Address(HIER_ADDR.clone())),
+            value: QueryComponent::Exact(HIER_ADDR.clone().into()),
         })))?;
 
     // TODO: this is horrible
@@ -144,7 +144,7 @@ pub fn fetch_or_create_dir(
         .query(Query::SingleQuery(QueryPart::Matches(EntryQuery {
             entity: QueryComponent::Any,
             attribute: QueryComponent::Exact(String::from(LABEL_ATTR)),
-            value: QueryComponent::Exact(EntryValue::String(directory.as_ref().clone())),
+            value: QueryComponent::Exact(directory.as_ref().clone().into()),
         })))?
         .into_iter()
         .map(|e: Entry| e.entity);
@@ -174,14 +174,14 @@ pub fn fetch_or_create_dir(
                 let type_entry = Entry {
                     entity: new_directory_address.clone(),
                     attribute: String::from(IS_OF_TYPE_ATTR),
-                    value: EntryValue::Address(HIER_ADDR.clone()),
+                    value: HIER_ADDR.clone().into(),
                 };
                 connection.insert_entry(type_entry)?;
 
                 let directory_entry = Entry {
                     entity: new_directory_address.clone(),
                     attribute: String::from(LABEL_ATTR),
-                    value: EntryValue::String(directory.as_ref().clone()),
+                    value: directory.as_ref().clone().into(),
                 };
                 connection.insert_entry(directory_entry)?;
 
@@ -189,7 +189,7 @@ pub fn fetch_or_create_dir(
                     let has_entry = Entry {
                         entity: parent,
                         attribute: String::from(HIER_HAS_ATTR),
-                        value: EntryValue::Address(new_directory_address.clone()),
+                        value: new_directory_address.clone().into(),
                     };
                     connection.insert_entry(has_entry)?;
                 }
