@@ -2,6 +2,7 @@ table! {
     data (identity) {
         identity -> Binary,
         entity -> Binary,
+        entity_searchable -> Nullable<Text>,
         attribute -> Text,
         value_str -> Nullable<Text>,
         value_num -> Nullable<Double>,
@@ -29,4 +30,8 @@ table! {
     }
 }
 
-allow_tables_to_appear_in_same_query!(data, files, meta,);
+allow_tables_to_appear_in_same_query!(
+    data,
+    files,
+    meta,
+);
