@@ -260,9 +260,9 @@ impl Query {
                                     q_entities.iter().map(|t| t.encode()).collect();
                                 subqueries.push(Box::new(data::entity.eq_any(entities?)))
                             }
-                            QueryComponent::Contains(_) => {
-                                return Err(anyhow!("Addresses cannot be queried alike."))
-                            }
+                            QueryComponent::Contains(q_entity) => subqueries.push(Box::new(
+                                data::entity_searchable.like(format!("%{}%", q_entity)),
+                            )),
                             QueryComponent::Any => {}
                         };
 
@@ -452,10 +452,7 @@ mod test {
         );
         query.to_sqlite_predicates()?;
 
-        let values = vec![
-            EntryValue::String("FOO".to_string()),
-            EntryValue::String("BAR".to_string()),
-        ];
+        let values: Vec<EntryValue> = vec!["FOO".into(), "BAR".into()];
         let query = format!(
             "(matches ? ? (in {}))",
             values
@@ -496,10 +493,7 @@ mod test {
         query.to_sqlite_predicates()?;
 
         // Invalid queries
-        let values = vec![
-            EntryValue::String("FOO".to_string()),
-            EntryValue::Number(1337.93),
-        ];
+        let values: Vec<EntryValue> = vec!["FOO".into(), EntryValue::Number(1337.93)];
         let query = format!(
             "(matches ? ? (in {}))",
             values
@@ -522,10 +516,7 @@ mod test {
             .err()
             .ok_or(anyhow!("Failed to reject mixed query."))?;
 
-        let values = vec![
-            EntryValue::Number(1337.93),
-            EntryValue::String("FOO".to_string()),
-        ];
+        let values = vec![EntryValue::Number(1337.93), "FOO".into()];
         let query = format!(
             "(matches ? ? (in {}))",
             values
@@ -553,6 +544,28 @@ mod test {
 
     #[test]
     fn test_contains() -> Result<()> {
+        let query = "(matches (contains \"foo\") ? ?)".parse::<Query>()?;
+        assert_eq!(
+            query,
+            Query::SingleQuery(QueryPart::Matches(EntryQuery {
+                entity: QueryComponent::Contains("foo".to_string()),
+                attribute: QueryComponent::Any,
+                value: QueryComponent::Any
+            }))
+        );
+        query.to_sqlite_predicates()?;
+
+        let query = "(matches ? (contains \"foo\") ?)".parse::<Query>()?;
+        assert_eq!(
+            query,
+            Query::SingleQuery(QueryPart::Matches(EntryQuery {
+                entity: QueryComponent::Any,
+                attribute: QueryComponent::Contains("foo".to_string()),
+                value: QueryComponent::Any,
+            }))
+        );
+        query.to_sqlite_predicates()?;
+
         let query = "(matches ? ? (contains \"foo\"))".parse::<Query>()?;
         assert_eq!(
             query,
