@@ -308,12 +308,12 @@ impl UpEndConnection {
         }
     }
 
-    pub fn retrieve_object(&self, object_address: Address) -> Result<Vec<Entry>> {
+    pub fn retrieve_object(&self, object_address: &Address) -> Result<Vec<Entry>> {
         use crate::database::inner::schema::data::dsl::*;
 
         let primary = data
             .filter(entity.eq(object_address.encode()?))
-            .or_filter(value_str.eq(EntryValue::Address(object_address).to_string()?))
+            .or_filter(value_str.eq(EntryValue::Address(object_address.clone()).to_string()?))
             .load::<models::Entry>(&self.conn)?;
 
         let entries = primary
