@@ -7,9 +7,10 @@ use std::{fs, iter};
 
 use crate::addressing::Address;
 use crate::database::constants::{
-    HIER_HAS_ATTR, IS_OF_TYPE_ATTR, LABEL_ATTR, TYPE_ADDR, TYPE_BASE_ATTR, TYPE_HAS_ATTR,
+    ADDED_ATTR, HIER_HAS_ATTR, IS_OF_TYPE_ATTR, LABEL_ATTR, TYPE_ADDR, TYPE_BASE_ATTR,
+    TYPE_HAS_ATTR,
 };
-use crate::database::entry::{Entry, EntryValue, InvariantEntry};
+use crate::database::entry::{Entry, InvariantEntry};
 use crate::database::hierarchies::{
     resolve_path, resolve_path_cached, ResolveCache, UHierPath, UNode,
 };
@@ -33,7 +34,7 @@ const FILE_SIZE_KEY: &str = "FILE_SIZE";
 lazy_static! {
     static ref BLOB_TYPE_INVARIANT: InvariantEntry = InvariantEntry {
         attribute: String::from(TYPE_BASE_ATTR),
-        value: EntryValue::String(String::from(BLOB_TYPE)),
+        value: BLOB_TYPE.into(),
     };
     static ref BLOB_TYPE_ADDR: Address = BLOB_TYPE_INVARIANT.entity().unwrap();
 }
@@ -397,30 +398,29 @@ fn insert_file_with_metadata(
     let type_entry = Entry {
         entity: blob_address.clone(),
         attribute: String::from(IS_OF_TYPE_ATTR),
-        value: EntryValue::Address(BLOB_TYPE_ADDR.clone()),
+        value: BLOB_TYPE_ADDR.clone().into(),
     };
 
     let size_entry = Entry {
         entity: blob_address.clone(),
         attribute: FILE_SIZE_KEY.to_string(),
-        value: EntryValue::Number(size as f64),
+        value: (size as f64).into(),
     };
 
     let mime_entry = mime_type.map(|mime_type| Entry {
         entity: blob_address.clone(),
         attribute: FILE_MIME_KEY.to_string(),
-        value: EntryValue::String(mime_type),
+        value: mime_type.into(),
     });
 
     let added_entry = Entry {
         entity: blob_address.clone(),
-        attribute: "ADDED".to_string(),
-        value: EntryValue::Number(
-            SystemTime::now()
-                .duration_since(UNIX_EPOCH)
-                .unwrap()
-                .as_secs() as f64,
-        ),
+        attribute: ADDED_ATTR.to_string(),
+        value: (SystemTime::now()
+            .duration_since(UNIX_EPOCH)
+            .unwrap()
+            .as_secs() as f64)
+            .into(),
     };
 
     // Add the appropriate entries w/r/t virtual filesystem location
@@ -456,21 +456,21 @@ fn insert_file_with_metadata(
         let dir_has_entry = Entry {
             entity: parent_dir.clone(),
             attribute: HIER_HAS_ATTR.to_string(),
-            value: EntryValue::Address(blob_address.clone()),
+            value: blob_address.clone().into(),
         };
         let dir_has_entry_addr = connection.insert_entry(dir_has_entry)?;
 
         let label_entry = Entry {
             entity: blob_address.clone(),
             attribute: LABEL_ATTR.to_string(),
-            value: EntryValue::String(filename.as_os_str().to_string_lossy().to_string()),
+            value: filename.as_os_str().to_string_lossy().to_string().into(),
         };
         let label_entry_addr = connection.insert_entry(label_entry)?;
 
         let alias_entry = Entry {
             entity: dir_has_entry_addr,
             attribute: ALIAS_KEY.to_string(),
-            value: EntryValue::Address(label_entry_addr),
+            value: label_entry_addr.into(),
         };
         connection.insert_entry(alias_entry)?;
 
