@@ -1,7 +1,9 @@
 use crate::util::hash::Hash;
+use crate::util::jobs::{Job, JobContainer, State};
 use crate::{database::UpEndDatabase, util::hash::b58_encode};
 use anyhow::{anyhow, Result};
 use log::{debug, trace};
+use std::sync::RwLock;
 use std::{
     collections::HashMap,
     fs::File,
@@ -52,7 +54,15 @@ impl PreviewStore {
         }
     }
 
-    pub fn get(&self, hash: Hash) -> Result<Option<PathBuf>> {
+    pub fn get<S>(
+        &self,
+        hash: Hash,
+        mime_type: S,
+        job_container: Arc<RwLock<JobContainer>>,
+    ) -> Result<Option<PathBuf>>
+    where
+        S: Into<Option<String>>,
+    {
         debug!("Preview for {hash:?} requested...");
         let path_mutex = self.get_path(&hash);
         let thumbpath = path_mutex.lock().unwrap();
@@ -64,30 +74,61 @@ impl PreviewStore {
             let connection = self.db.connection()?;
             let files = connection.retrieve_file(&hash)?;
             if let Some(file) = files.get(0) {
-                let preview = match tree_magic_mini::from_filepath(&file.path) {
-                    Some(tm) if tm.starts_with("text") => Ok(TextPath(&file.path).get_thumbnail()?),
+                let job_id = job_container
+                    .write()
+                    .unwrap()
+                    .add_job(Job::new(
+                        None,
+                        &format!("Creating preview for {:?}", file.path.file_name().unwrap()),
+                    ))
+                    .unwrap();
+
+                let mime_type = mime_type.into();
+
+                let mime_type: Option<String> = if mime_type.is_some() {
+                    mime_type
+                } else {
+                    tree_magic_mini::from_filepath(&file.path).map(|m| m.into())
+                };
+
+                let preview = match mime_type {
+                    Some(tm) if tm.starts_with("text") => TextPath(&file.path).get_thumbnail(),
                     Some(tm) if tm.starts_with("video") || tm == "application/x-matroska" => {
-                        Ok(VideoPath(&file.path).get_thumbnail()?)
+                        VideoPath(&file.path).get_thumbnail()
                     }
-                    Some(tm) if tm.starts_with("audio") => {
-                        Ok(AudioPath(&file.path).get_thumbnail()?)
-                    }
-                    Some(tm) if tm.starts_with("image") => {
-                        Ok(ImagePath(&file.path).get_thumbnail()?)
+                    Some(tm) if tm.starts_with("audio") || tm == "application/x-riff" => {
+                        AudioPath(&file.path).get_thumbnail()
                     }
+                    Some(tm) if tm.starts_with("image") => ImagePath(&file.path).get_thumbnail(),
                     Some(unknown) => Err(anyhow!("No capability for {:?} thumbnails.", unknown)),
                     _ => Err(anyhow!("Unknown file type, or file doesn't exist.")),
-                }?;
+                };
 
-                trace!("Got preview for {hash:?}.");
+                match preview {
+                    Ok(preview) => {
+                        trace!("Got preview for {hash:?}.");
 
-                if let Some(data) = preview {
-                    std::fs::create_dir_all(&self.path)?;
-                    let mut file = File::create(&*thumbpath)?;
-                    file.write_all(&data)?;
-                    Ok(Some(thumbpath.clone()))
-                } else {
-                    Ok(None)
+                        let _ = job_container
+                            .write()
+                            .unwrap()
+                            .update_state(&job_id, State::Done);
+
+                        if let Some(data) = preview {
+                            std::fs::create_dir_all(&self.path)?;
+                            let mut file = File::create(&*thumbpath)?;
+                            file.write_all(&data)?;
+                            Ok(Some(thumbpath.clone()))
+                        } else {
+                            Ok(None)
+                        }
+                    }
+                    Err(err) => {
+                        let _ = job_container
+                            .write()
+                            .unwrap()
+                            .update_state(&job_id, State::Failed);
+                        Err(err)
+                    }
                 }
             } else {
                 Err(anyhow!("Object not found, or is not a file."))
