@@ -1,8 +1,10 @@
 use crate::addressing::{Address, Addressable};
-use crate::database::entry::{Entry, EntryValue, InEntry};
+use crate::database::constants::{ADDED_ATTR, LABEL_ATTR};
+use crate::database::entry::{Entry, EntryValue, InvariantEntry};
 use crate::database::hierarchies::{list_roots, resolve_path, UHierPath};
 use crate::database::lang::Query;
 use crate::database::UpEndDatabase;
+use crate::extractors::Extractor;
 use crate::filesystem::add_file;
 use crate::previews::PreviewStore;
 use crate::util::hash::{b58_decode, b58_encode, Hashable};
@@ -10,21 +12,23 @@ use crate::util::jobs::JobContainer;
 use actix_files::NamedFile;
 use actix_multipart::Multipart;
 use actix_web::error::{ErrorBadRequest, ErrorInternalServerError, ErrorNotFound};
-use actix_web::http;
-use actix_web::http::header::{ContentDisposition, DispositionType};
+use actix_web::http::header::{CacheControl, CacheDirective, ContentDisposition, DispositionType};
 use actix_web::{delete, error, get, post, put, web, Either, Error, HttpResponse};
+use actix_web::{http, Responder};
 use anyhow::{anyhow, Result};
 use futures_util::TryStreamExt;
 use log::{debug, info, trace};
 use serde::Deserialize;
 use serde_json::json;
-use std::convert::TryFrom;
+use std::convert::{TryFrom, TryInto};
 use std::fs;
 use std::io::Write;
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
+use std::time::{SystemTime, UNIX_EPOCH};
 use std::{collections::HashMap, io};
 use tempfile::NamedTempFile;
+use uuid::Uuid;
 
 #[cfg(feature = "desktop")]
 use is_executable::IsExecutable;
@@ -49,7 +53,7 @@ pub async fn get_raw(
     state: web::Data<State>,
     web::Query(query): web::Query<RawRequest>,
     hash: web::Path<String>,
-) -> Result<Either<NamedFile, HttpResponse>, Error> {
+) -> Result<impl Responder, Error> {
     let address =
         Address::decode(&b58_decode(hash.into_inner()).map_err(ErrorInternalServerError)?)
             .map_err(ErrorInternalServerError)?;
@@ -73,14 +77,22 @@ pub async fn get_raw(
 
             if query.native.is_none() {
                 Ok(Either::A(
-                    NamedFile::open(file_path)?.set_content_disposition(ContentDisposition {
-                        disposition: if query.inline.is_some() {
-                            DispositionType::Inline
-                        } else {
-                            DispositionType::Attachment
-                        },
-                        parameters: vec![],
-                    }),
+                    NamedFile::open(file_path)?
+                        .set_content_disposition(ContentDisposition {
+                            disposition: if query.inline.is_some() {
+                                DispositionType::Inline
+                            } else {
+                                DispositionType::Attachment
+                            },
+                            parameters: vec![],
+                        })
+                        .with_header(
+                            http::header::CACHE_CONTROL,
+                            CacheControl(vec![
+                                CacheDirective::MaxAge(2678400),
+                                CacheDirective::Extension("immutable".into(), None),
+                            ]),
+                        ),
                 ))
             } else if state.desktop_enabled {
                 #[cfg(feature = "desktop")]
@@ -126,10 +138,16 @@ pub async fn get_raw(
     }
 }
 
+#[derive(Deserialize)]
+pub struct ThumbRequest {
+    mime: Option<String>,
+}
+
 #[get("/api/thumb/{hash}")]
 pub async fn get_thumbnail(
     state: web::Data<State>,
     hash: web::Path<String>,
+    web::Query(query): web::Query<ThumbRequest>,
 ) -> Result<Either<NamedFile, HttpResponse>, Error> {
     #[cfg(feature = "previews")]
     if let Some(preview_store) = &state.preview_store {
@@ -138,7 +156,10 @@ pub async fn get_thumbnail(
             .map_err(ErrorInternalServerError)?;
         if let Address::Hash(address_hash) = address {
             let preview_store = preview_store.clone();
-            let preview_result = web::block(move || preview_store.get(address_hash)).await?;
+            let preview_result = web::block(move || {
+                preview_store.get(address_hash, query.mime, state.job_container.clone())
+            })
+            .await?;
 
             if let Some(preview_path) = preview_result {
                 let mut file = NamedFile::open(&preview_path)?.disable_content_disposition();
@@ -151,7 +172,7 @@ pub async fn get_thumbnail(
             } else {
                 return Ok(Either::B(
                     HttpResponse::SeeOther()
-                        .header(http::header::LOCATION, format!("/api/raw/{hash}"))
+                        .header(http::header::LOCATION, format!("../../api/raw/{hash}"))
                         .finish(),
                 ));
             }
@@ -207,18 +228,70 @@ impl EntriesAsHash for Vec<Entry> {
 #[get("/api/obj/{address_str}")]
 pub async fn get_object(
     state: web::Data<State>,
-    address_str: web::Path<String>,
+    address: web::Path<Address>,
 ) -> Result<HttpResponse, Error> {
     let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
+    let address = address.into_inner();
     let result: Vec<Entry> = connection
-        .retrieve_object(
-            Address::decode(&b58_decode(address_str.into_inner()).map_err(ErrorBadRequest)?)
-                .map_err(ErrorBadRequest)?,
-        )
+        .retrieve_object(&address)
         .map_err(ErrorInternalServerError)?;
 
     debug!("{:?}", result);
-    Ok(HttpResponse::Ok().json(result.as_hash().map_err(ErrorInternalServerError)?))
+
+    // TODO: make this automatically derive from `Address` definition
+    let (entity_type, entity_content) = match address {
+        Address::Hash(_) => ("Hash", None),
+        Address::Uuid(_) => ("Uuid", None),
+        Address::Attribute(attribute) => ("Attribute", Some(attribute)),
+        Address::Url(url) => ("Url", Some(url)),
+    };
+
+    Ok(HttpResponse::Ok().json(json!({
+        "entity": {
+            "t": entity_type,
+            "c": entity_content
+        },
+        "entries": result.as_hash().map_err(ErrorInternalServerError)?
+    })))
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[serde(untagged)]
+pub enum InEntry {
+    Entry(Entry),
+    EntryList(Vec<Entry>),
+    Invariant(InvariantEntry),
+    Address { entity: InAddress },
+}
+
+#[derive(Debug, Clone, Deserialize)]
+#[serde(untagged)]
+pub enum InAddress {
+    Address(String),
+    Components { t: String, c: Option<String> },
+}
+
+impl TryInto<Address> for InAddress {
+    type Error = anyhow::Error;
+
+    fn try_into(self) -> Result<Address, Self::Error> {
+        Ok(match self {
+            InAddress::Address(address) => address.parse()?,
+            InAddress::Components { t, c } => {
+                // I absolutely cannot handle serde magic right now
+                // TODO: make this automatically derive from `Address` definition
+                match t.as_str() {
+                    "Attribute" => Address::Attribute(c.ok_or(anyhow!("Missing attribute."))?),
+                    "Url" => Address::Url(c.ok_or(anyhow!("Missing URL."))?),
+                    "Uuid" => match c {
+                        Some(c) => c.parse()?,
+                        None => Address::Uuid(Uuid::new_v4()),
+                    },
+                    _ => c.ok_or(anyhow!("Missing address."))?.parse()?,
+                }
+            }
+        })
+    }
 }
 
 #[put("/api/obj")]
@@ -226,19 +299,103 @@ pub async fn put_object(
     state: web::Data<State>,
     payload: Either<web::Json<InEntry>, Multipart>,
 ) -> Result<HttpResponse, Error> {
-    let (result_address, entry) = match payload {
+    let (entry_address, entity_address) = match payload {
         Either::A(in_entry) => {
-            let in_entry = in_entry.into_inner();
-            let entry = Entry::try_from(in_entry).map_err(ErrorInternalServerError)?;
-
             let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
+            let in_entry = in_entry.into_inner();
 
-            Ok((
-                connection
-                    .insert_entry(entry.clone())
-                    .map_err(ErrorInternalServerError)?,
-                Some(entry),
-            ))
+            debug!("PUTting {in_entry:?}");
+
+            match in_entry {
+                InEntry::Entry(entry) => Ok((
+                    Some(
+                        connection
+                            .insert_entry(entry.clone())
+                            .map_err(ErrorInternalServerError)?,
+                    ),
+                    Some(entry.entity),
+                )),
+                InEntry::EntryList(entries) => {
+                    connection
+                        .transaction::<_, anyhow::Error, _>(|| {
+                            for entry in entries {
+                                connection.insert_entry(entry)?;
+                            }
+                            Ok(())
+                        })
+                        .map_err(ErrorInternalServerError)?;
+                    Ok((None, None))
+                }
+                InEntry::Invariant(in_entry) => {
+                    let invariant = Entry::try_from(&InvariantEntry {
+                        attribute: in_entry.attribute,
+                        value: in_entry.value,
+                    })
+                    .map_err(ErrorInternalServerError)?;
+
+                    Ok((
+                        Some(
+                            connection
+                                .insert_entry(invariant.clone())
+                                .map_err(ErrorInternalServerError)?,
+                        ),
+                        Some(invariant.entity),
+                    ))
+                }
+                InEntry::Address { entity: in_address } => {
+                    let address = in_address.try_into().map_err(ErrorBadRequest)?;
+
+                    let entries_to_add = match &address {
+                        Address::Hash(_) | Address::Uuid(_) => vec![],
+                        Address::Attribute(attribute) => vec![Entry {
+                            entity: address.clone(),
+                            attribute: LABEL_ATTR.to_string(),
+                            value: format!("ATTRIBUTE: {attribute}").into(),
+                        }],
+                        Address::Url(url) => {
+                            #[cfg(feature = "extractors-web")]
+                            actix::spawn(
+                                (crate::extractors::web::WebExtractor {}).insert_info_fnf(
+                                    address.clone(),
+                                    state.upend.connection().map_err(ErrorInternalServerError)?,
+                                    state.job_container.clone(),
+                                ),
+                            );
+                            vec![Entry {
+                                entity: address.clone(),
+                                attribute: LABEL_ATTR.to_string(),
+                                value: url.clone().into(),
+                            }]
+                        }
+                    };
+
+                    connection
+                        .transaction::<_, anyhow::Error, _>(|| {
+                            if connection.retrieve_object(&address)?.is_empty() {
+                                connection.insert_entry(Entry {
+                                    entity: address.clone(),
+                                    attribute: ADDED_ATTR.to_string(),
+                                    value: EntryValue::Number(
+                                        SystemTime::now()
+                                            .duration_since(UNIX_EPOCH)
+                                            .unwrap()
+                                            .as_secs()
+                                            as f64,
+                                    ),
+                                })?;
+                            }
+
+                            for entry in entries_to_add {
+                                connection.insert_entry(entry)?;
+                            }
+
+                            Ok(())
+                        })
+                        .map_err(ErrorInternalServerError)?;
+
+                    Ok((None, Some(address)))
+                }
+            }
         }
         Either::B(mut multipart) => {
             if let Some(mut field) = multipart.try_next().await? {
@@ -284,25 +441,18 @@ pub async fn put_object(
                 }
 
                 if let Some(filename) = filename {
-                    let _ = upend_insert_val!(&connection, address, "LBL", filename);
+                    let _ = upend_insert_val!(&connection, address, LABEL_ATTR, filename);
                 }
 
-                Ok((address, None))
+                Ok((None, Some(address)))
             } else {
-                Err(ErrorBadRequest(anyhow!("wat")))
+                Err(anyhow!("Multipart contains no fields."))
             }
         }
-    }?;
+    }
+    .map_err(ErrorInternalServerError)?;
 
-    Ok(HttpResponse::Ok().json(
-        [(
-            b58_encode(result_address.encode().map_err(ErrorInternalServerError)?),
-            entry,
-        )]
-        .iter()
-        .cloned()
-        .collect::<HashMap<String, Option<Entry>>>(),
-    ))
+    Ok(HttpResponse::Ok().json([entry_address, entity_address]))
 }
 
 #[put("/api/obj/{address}/{attribute}")]
@@ -393,7 +543,7 @@ pub async fn list_hier(
     let connection = state.upend.connection().map_err(ErrorInternalServerError)?;
     if path.is_empty() {
         Ok(HttpResponse::MovedPermanently()
-            .header("Location", "/api/hier_roots")
+            .header(http::header::LOCATION, "../../api/hier_roots")
             .finish())
     } else {
         let upath: UHierPath = path.into_inner().parse().map_err(ErrorBadRequest)?;
@@ -403,7 +553,7 @@ pub async fn list_hier(
         let path = resolve_path(&connection, &upath, false).map_err(ErrorNotFound)?;
         match path.last() {
             Some(addr) => Ok(HttpResponse::Found()
-                .header("Location", format!("/api/obj/{}", addr))
+                .header(http::header::LOCATION, format!("../../api/obj/{}", addr))
                 .finish()),
             None => Ok(HttpResponse::NotFound().finish()),
         }
@@ -417,7 +567,7 @@ pub async fn list_hier_roots(state: web::Data<State>) -> Result<HttpResponse, Er
     let result = list_roots(&connection)
         .map_err(ErrorInternalServerError)?
         .into_iter()
-        .map(|root| connection.retrieve_object(root))
+        .map(|root| connection.retrieve_object(&root))
         .collect::<Result<Vec<Vec<Entry>>>>()
         .map_err(ErrorInternalServerError)?
         .concat();
@@ -474,10 +624,24 @@ pub async fn latest_files(state: web::Data<State>) -> Result<HttpResponse, Error
     Ok(HttpResponse::Ok().json(&files))
 }
 
+#[derive(Deserialize)]
+pub struct JobsRequest {
+    full: Option<String>,
+}
+
 #[get("/api/jobs")]
-pub async fn get_jobs(state: web::Data<State>) -> Result<HttpResponse, Error> {
+pub async fn get_jobs(
+    state: web::Data<State>,
+    web::Query(query): web::Query<JobsRequest>,
+) -> Result<HttpResponse, Error> {
     let jobs = state.job_container.read().unwrap().get_jobs();
-    Ok(HttpResponse::Ok().json(&jobs))
+    Ok(HttpResponse::Ok().json(if query.full.is_some() {
+        jobs
+    } else {
+        jobs.into_iter()
+            .filter(|(_, j)| matches!(j.state, crate::util::jobs::State::InProgress))
+            .collect()
+    }))
 }
 
 #[get("/api/info")]
@@ -489,3 +653,24 @@ pub async fn get_info(state: web::Data<State>) -> Result<HttpResponse, Error> {
         "desktop": state.desktop_enabled
     })))
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use anyhow::Result;
+
+    #[test]
+    fn test_in_address() -> Result<()> {
+        let address = Address::Url("https://upendproject.net".into());
+        let in_address = InAddress::Address(address.to_string());
+        assert_eq!(address, in_address.try_into()?);
+
+        let in_address = InAddress::Components {
+            t: "Url".into(),
+            c: Some("https://upendproject.net".into()),
+        };
+        assert_eq!(address, in_address.try_into()?);
+
+        Ok(())
+    }
+}
