@@ -20,15 +20,37 @@ export type IValue =
       t: "Number";
       c: number;
     }
+  | {
+      t: "Null";
+      c: null;
+    }
   | {
       t: "Invalid";
       c: null;
     };
 
+export interface InvariantEntry {
+  attribute: string;
+  value: IValue;
+}
+
+export type InAddress =
+  | Address
+  | { t: "Attribute" | "Url" | "Uuid"; c?: string };
+
+export type InEntry =
+  | IEntry
+  | IEntry[]
+  | InvariantEntry
+  | { entity: InAddress };
+
 export interface ListingResult {
   [key: string]: IEntry;
 }
 
+// entry address, entity address address
+export type PutResult = [string | undefined, string];
+
 // export type OrderedListing = [Address, IEntry][];
 
 export interface IFile {
