@@ -1,8 +1,9 @@
 import { readable, Readable } from "svelte/store";
 import type { VaultInfo } from "upend/types";
+import { fetchInfo } from "../lib/api";
 
 export const vaultInfo: Readable<VaultInfo> = readable(undefined, (set) => {
-  fetch("api/info").then(async (response) => {
-    set(await response.json());
+  fetchInfo().then(async (info) => {
+    set(info);
   });
 });
